/*************************************************************************
*	TG300 MULTI PART
*************************************************************************/

int		Tg300partAdrOfs[16] = 
{
	0x100,0x200,0x300,0x400,0x500,0x600,0x700,0x800,0x900,0x000,0xA00,0xB00,0xC00,0xD00,0xE00,0xF00
};

int		tg300_getPart( char *s )
{
	int		part;

	part = atoi(s) - 1;
	if ( part < 0 || part > 15 )
		return (-1);
	return (part);
}

/*************************************************************************
*	ELEMENT RESERVE
*-------------------------------------------------------------------------
*	exTG300_elementReserve(part,n)
*		part	c	1 ` 16
*************************************************************************/

int		exTG300_elementReserve( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 32 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020000+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"ELEMENT RESERVE\" error!");
	return (-1);
}


/*************************************************************************
*	BANK SELECT
*-------------------------------------------------------------------------
*	exTG300_bank(part,msb,lsb)
*		part	c	1 ` 16
*************************************************************************/

int		exTG300_bank( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, msb, lsb;

	if ( argc != 3 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	msb = atoi(argv[1]);	if ( msb < 0 || msb > 0x7F )	goto _ERR;
	lsb = atoi(argv[2]);	if ( lsb < 0 || lsb > 0x7F )	goto _ERR;
	dat[0] = msb;
	dat[1] = lsb;
	return tg300_genMML( sqr, bufSiz, buf, 0x020001+Tg300partAdrOfs[part], 2, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"BANK SELECT\" error!");
	return (-1);
}


/*************************************************************************
*	PROGRAM NUMBER
*-------------------------------------------------------------------------
*	exTG300_programNo(part,prog)
*		part	c	1 ` 16
*		prog	c	1`128
*************************************************************************/

int		exTG300_programNo( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]) - 1;
	if ( v < 0 || v > 127 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020003+Tg300partAdrOfs[part], 2, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"PROGRAM NUMBER\" error!");
	return (-1);
}

/*************************************************************************
*	Rcv CHANNEL
*-------------------------------------------------------------------------
*	exTG300_rcvChannel(part,ch)
*	part	c	1 ` 16
*	ch		c	0`16
*************************************************************************/

int		exTG300_rcvChannel( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v == 0 )
		v = 0x10;	/* OFF */
	else
		--v;
	if ( v < 0 || v > 16 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020004+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"Rcv CHANNEL\" error!");
	return (-1);
}

/*************************************************************************
*	MONO/POLY MODE
*-------------------------------------------------------------------------
*	exTG300_monoPolyMode(part,mode)
*	part	c	1 ` 16
*	mode	c	0:MONO
*				1:POLY
*************************************************************************/

int		exTG300_monoPolyMode( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 1 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020015+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"MONO/POLY MODE\" error!");
	return(-1);
}

/*************************************************************************
*	SAME NOTE NUMBER KEY ON ASSIGN
*-------------------------------------------------------------------------
*	exTG300_keyAssign(part,mode)
*	part	c	1 ` 16
*	mode	c	0:SINGLE
*				1:MULTI
*				2:INST
*************************************************************************/

int		exTG300_keyAssign( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 2 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020016+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"SAME NOTE NUMBER KEY ON ASSIGN\" error!");
	return (-1);
}

/*************************************************************************
*	PART MODE
*-------------------------------------------------------------------------
*	exTG300_partMode(part,mode)
*		part	c	1 ` 16
*		mode	c	0:NORMAL
*					1:DRUM
*************************************************************************/

int		exTG300_partMode( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 1 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020017+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"PART MODE\" error!");
	return (-1);
}

/*************************************************************************
*	NOTE SHIFT
*-------------------------------------------------------------------------
*	exTG300_noteShift(part,shift)
*		part	c	1 ` 16
*		shift	c	-24`+24
*************************************************************************/

int		exTG300_noteShift( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]) + 0x40;
	if ( v < 0x28 || v > 0x58 )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x020018+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"NOTE SHIFT\" error!");
	return (-1);
}

/*************************************************************************
*	DETUNE
*-------------------------------------------------------------------------
*	exTG300_detune(part,shift)
*	part	c	1 ` 16
*	shift	c	-128`+127
*************************************************************************/

int		exTG300_detune( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]) + 0x80;
	if ( v < 0x00 || v > 0xFF )
		goto _ERR;
	dat[0] = (v >> 4) & 0x0F;
	dat[1] =  v       & 0x0F;
	return tg300_genMML( sqr, bufSiz, buf, 0x020019+Tg300partAdrOfs[part], 2, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"DETUNE\" error!");
	return (-1);
}

/*************************************************************************
*	VOLUME
*-------------------------------------------------------------------------
*	exTG300_volume(part,vol)
*		part	c	1 ` 16
*		vol		c	0 `127
*************************************************************************/

int		exTG300_volume( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 0x7F )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x02001B+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"VOLUME\" error!");
	return (-1);
}


/*************************************************************************
*	VELOCITY SENSE DEPTH
*-------------------------------------------------------------------------
*	exTG300_velSensDepth(part,depth)
*		part	c	1 ` 16
*		depth	c	0 `127
*************************************************************************/

int		exTG300_velSensDepth( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 0x7F )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x02001C+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"VELOCITY SENSE DEPTH\" error!");
	return (-1);
}

/*************************************************************************
*	VELOCITY SENSE OFFSET
*-------------------------------------------------------------------------
*	exTG300_velSensOffst(part,offset)
*		part	c	1 ` 16
*		offset	c	0 `127
*************************************************************************/

int		exTG300_velSensOffst( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	v = atoi(argv[1]);
	if ( v < 0 || v > 0x7F )
		goto _ERR;
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x02001D+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"VELOCITY SENSE OFFSET\" error!");
	return (-1);
}

/*************************************************************************
*	PAN
*-------------------------------------------------------------------------
*	exTG300_pan(part,pan)
*		part	c	1 ` 16
*		pan		c	rnd, -63`0`63
*************************************************************************/

int		exTG300_pan( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[4];
	int		part, v;

	if ( argc != 2 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	if ( strcmpi(argv[1],"rnd") == 0 )
		v = 0;
	else
	{
		v = atoi(argv[1]);
		if ( v < -63 || v > 63 )
			goto _ERR;
		v += 64;
	}
	dat[0] = v;
	return tg300_genMML( sqr, bufSiz, buf, 0x02001E+Tg300partAdrOfs[part], 1, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"PAN\" error!");
	return (-1);
}


/*************************************************************************
*	MW CONTROL
*-------------------------------------------------------------------------
*	exTG300_mwControl(part,pitch,filter,amp,pmod,fmod)
*		part	c	1 ` 16
*		pitch	c	-24`24
*		filter	c	-64`63
*		amp		c	-64`63
*		pmod	c	0`127
*		fmod	c	0`127
*************************************************************************/

int		exTG300_mwControl( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[8];
	int		part, pitch, filter, amp, pmod, fmod;

	if ( argc != 6 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	pitch  = atoi(argv[1]) + 0x40;
	filter = atoi(argv[2]) + 0x40;
	amp    = atoi(argv[3]) + 0x40;
	pmod   = atoi(argv[4]);
	fmod   = atoi(argv[5]);
	if ( pitch < 0x28 || pitch > 0x58 || filter < 0 || filter > 0x7F
	|| amp < 0 || amp > 0x7F || pmod < 0 || pmod > 0x7F || fmod <  0 || fmod > 0x7F )
		goto _ERR;
	dat[0] = pitch;
	dat[1] = filter;
	dat[2] = amp;
	dat[3] = pmod;
	dat[4] = fmod;
	return tg300_genMML( sqr, bufSiz, buf, 0x02003B+Tg300partAdrOfs[part], 5, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"MW CONTROL\" error!");
	return (-1);
}


/*************************************************************************
*	BEND CONTROL
*-------------------------------------------------------------------------
*	exTG300_bendControl(part,pitch,filter,amp,pmod,fmod)
*		part	c	1 ` 16
*		pitch	c	-24`24
*		filter	c	-64`63
*		amp		c	-64`63
*		pmod	c	0`127
*		fmod	c	0`127
*************************************************************************/

int		exTG300_bendControl( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[8];
	int		part, pitch, filter, amp, pmod, fmod;

	if ( argc != 6 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	pitch  = atoi(argv[1]) + 0x40;
	filter = atoi(argv[2]) + 0x40;
	amp    = atoi(argv[3]) + 0x40;
	pmod   = atoi(argv[4]);
	fmod   = atoi(argv[5]);
	if ( pitch < 0x28 || pitch > 0x58 || filter < 0 || filter > 0x7F
	|| amp < 0 || amp > 0x7F || pmod < 0 || pmod > 0x7F || fmod <  0 || fmod > 0x7F )
		goto _ERR;
	dat[0] = pitch;
	dat[1] = filter;
	dat[2] = amp;
	dat[3] = pmod;
	dat[4] = fmod;
	return tg300_genMML( sqr, bufSiz, buf, 0x020041+Tg300partAdrOfs[part], 5, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"BEND CONTROL\" error!");
	return (-1);
}

/*************************************************************************
*	CAT CONTROL
*-------------------------------------------------------------------------
*	exTG300_catControl(part,pitch,filter,amp,pmod,fmod)
*		part	c	1 ` 16
*		pitch	c	-24`24
*		filter	c	-64`63
*		amp		c	-64`63
*		pmod	c	0`127
*		fmod	c	0`127
*************************************************************************/

int		exTG300_catControl( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[8];
	int		part, pitch, filter, amp, pmod, fmod;

	if ( argc != 6 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	pitch  = atoi(argv[1]) + 0x40;
	filter = atoi(argv[2]) + 0x40;
	amp    = atoi(argv[3]) + 0x40;
	pmod   = atoi(argv[4]);
	fmod   = atoi(argv[5]);
	if ( pitch < 0x28 || pitch > 0x58 || filter < 0 || filter > 0x7F
	|| amp < 0 || amp > 0x7F || pmod < 0 || pmod > 0x7F || fmod <  0 || fmod > 0x7F )
		goto _ERR;
	dat[0] = pitch;
	dat[1] = filter;
	dat[2] = amp;
	dat[3] = pmod;
	dat[4] = fmod;
	return tg300_genMML( sqr, bufSiz, buf, 0x020047+Tg300partAdrOfs[part], 5, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"CAT CONTROL\" error!");
	return (-1);
}

/*************************************************************************
*	PAT CONTROL
*-------------------------------------------------------------------------
*	exTG300_patControl(part,pitch,filter,amp,pmod,fmod)
*		part	c	1 ` 16
*		pitch	c	-24`24
*		filter	c	-64`63
*		amp		c	-64`63
*		pmod	c	0`127
*		fmod	c	0`127
*************************************************************************/

int		exTG300_patControl( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[8];
	int		part, pitch, filter, amp, pmod, fmod;

	if ( argc != 6 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	pitch  = atoi(argv[1]) + 0x40;
	filter = atoi(argv[2]) + 0x40;
	amp    = atoi(argv[3]) + 0x40;
	pmod   = atoi(argv[4]);
	fmod   = atoi(argv[5]);
	if ( pitch < 0x28 || pitch > 0x58 || filter < 0 || filter > 0x7F
	|| amp < 0 || amp > 0x7F || pmod < 0 || pmod > 0x7F || fmod <  0 || fmod > 0x7F )
		goto _ERR;
	dat[0] = pitch;
	dat[1] = filter;
	dat[2] = amp;
	dat[3] = pmod;
	dat[4] = fmod;
	return tg300_genMML( sqr, bufSiz, buf, 0x02004D+Tg300partAdrOfs[part], 5, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"PAT CONTROL\" error!");
	return (-1);
}

/*************************************************************************
*	AC1 CONTROL
*-------------------------------------------------------------------------
*	exTG300_ac1Control(part,pitch,filter,amp,pmod,fmod)
*		part	c	1 ` 16
*		pitch	c	-24`24
*		filter	c	-64`63
*		amp		c	-64`63
*		pmod	c	0`127
*		fmod	c	0`127
*************************************************************************/

int		exTG300_ac1Control( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[8];
	int		part, pitch, filter, amp, pmod, fmod;

	if ( argc != 6 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	pitch  = atoi(argv[1]) + 0x40;
	filter = atoi(argv[2]) + 0x40;
	amp    = atoi(argv[3]) + 0x40;
	pmod   = atoi(argv[4]);
	fmod   = atoi(argv[5]);
	if ( pitch < 0x28 || pitch > 0x58 || filter < 0 || filter > 0x7F
	|| amp < 0 || amp > 0x7F || pmod < 0 || pmod > 0x7F || fmod <  0 || fmod > 0x7F )
		goto _ERR;
	dat[0] = pitch;
	dat[1] = filter;
	dat[2] = amp;
	dat[3] = pmod;
	dat[4] = fmod;
	return tg300_genMML( sqr, bufSiz, buf, 0x020053+Tg300partAdrOfs[part], 5, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"AC1 CONTROL\" error!");
	return (-1);
}

/*************************************************************************
*	AC2 CONTROL
*-------------------------------------------------------------------------
*	exTG300_ac2Control(part,pitch,filter,amp,pmod,fmod)
*		part	c	1 ` 16
*		pitch	c	-24`24
*		filter	c	-64`63
*		amp		c	-64`63
*		pmod	c	0`127
*		fmod	c	0`127
*************************************************************************/

int		exTG300_ac2Control( char *sqr, size_t bufSiz, char *buf, int argc, char **argv )
{
	char	dat[8];
	int		part, pitch, filter, amp, pmod, fmod;

	if ( argc != 6 )
		goto _ERR;
	if ( (part = tg300_getPart(argv[0])) < 0 )
		goto _ERR;
	pitch  = atoi(argv[1]) + 0x40;
	filter = atoi(argv[2]) + 0x40;
	amp    = atoi(argv[3]) + 0x40;
	pmod   = atoi(argv[4]);
	fmod   = atoi(argv[5]);
	if ( pitch < 0x28 || pitch > 0x58 || filter < 0 || filter > 0x7F
	|| amp < 0 || amp > 0x7F || pmod < 0 || pmod > 0x7F || fmod <  0 || fmod > 0x7F )
		goto _ERR;
	dat[0] = pitch;
	dat[1] = filter;
	dat[2] = amp;
	dat[3] = pmod;
	dat[4] = fmod;
	return tg300_genMML( sqr, bufSiz, buf, 0x020059+Tg300partAdrOfs[part], 5, dat );

_ERR:
	MM_setErrMsg(sqr,"TG300 \"AC2 CONTROL\" error!");
	return (-1);
}

